@@ -2,13 +2,12 @@ package SMS::Send::Adapter::Node::Red;
 use strict;
 use warnings;
 use base qw{Package::New};
-use JSON qw{decode_json encode_json};
+use JSON::XS qw{decode_json encode_json};
 use SMS::Send;
 use CGI;
 
-our $VERSION = '0.08';
+our $VERSION = '0.10';
 our $PACKAGE = __PACKAGE__;
-our $DRIVER  = $ENV{'SMS_SEND_ADAPTER_NODE_RED_DRIVER'};
 
 =head1 NAME
 
@@ -169,6 +168,36 @@ sub set_status_error {
   return $self;
 }
 
+=head2 driver
+
+Returns configured SMS driver from input, environment, or SMS-Send.ini.
+
+=cut
+
+sub driver {
+  my $self     = shift;
+  my $driver   = undef;
+  my $ini_file = '/etc/SMS-Send.ini';
+  my $input    = $self->input; #undef on error
+  if (defined $input) {
+    #Set driver from input
+    $driver    = $input->{'driver'} if $input->{'driver'};
+  }
+  if (!$driver) {
+    #Set driver from environment
+    my $DRIVER = $ENV{'SMS_SEND_ADAPTER_NODE_RED_DRIVER'};
+    $driver    = $DRIVER if $DRIVER;
+  }
+  if (!$driver and -r $ini_file) {
+    #Set driver from INI file
+    require Config::IniFiles;
+    my $cfg     = Config::IniFiles->new(-file=>$ini_file);
+    my @drivers = grep {$cfg->val($_, 'active', '1')} $cfg->Sections;
+    $driver     = $drivers[0] if @drivers;
+  }
+  return $driver;
+}
+
 =head1 METHODS (ACTIONS)
 
 =head2 send_sms
@@ -276,10 +305,10 @@ Returns a L<SMS::Send> object for use in this package.
 =cut
 
 sub SMS {
-  my $self   = shift;
-  my $input  = $self->input; #undef on error
+  my $self  = shift;
+  my $input = $self->input; #undef on error
   if (defined $input) {
-    my $driver = $input->{'driver'} || $DRIVER;
+    my $driver   = $self->driver;
     if ($driver) {
       my $options = $input->{'options'} || {};
       if (ref($options) eq 'HASH') {
@@ -287,14 +316,14 @@ sub SMS {
         $self->{'SMS'} =  eval{SMS::Send->new($driver, %$options)};
         my $error      = $@;
         if ($error) {
-          my $text = qq{Failed to load Perl package SMS::Send with driver "$driver". Please ensure both SMS::Send and SMS::Send::$driver are installed. $error};
+          my $text = qq{Failed to load Perl package SMS::Send with driver "$driver". Ensure SMS::Send::$driver is installed. $error};
           $self->set_status_error(500=>$text);
         }
       } else {
         $self->set_status_error(400=>'Error: JSON input "options" not an object.');
       }
     } else {
-      $self->set_status_error(400=>'Error: JSON input missing "driver".');
+      $self->set_status_error(400=>'Error: "driver" not defined in JSON payload, environment variable SMS_SEND_ADAPTER_NODE_RED_DRIVER, or in SMS-Send.ini.');
     }
   }
   return $self->{'SMS'};
@@ -302,7 +331,7 @@ sub SMS {
 
 =head1 SEE ALSO
 
-L<SMS::Send>, L<CGI>, L<JSON>
+L<SMS::Send>, L<CGI>, L<JSON::XS>
 
 =head1 AUTHOR
 
